# This dialog was created using MakePerlQtForm.bsh
# Use DesignPerlQtForm.bsh to edit the form and automatically rebuild source files.

package VeSql;
use Qt;
use DBI;
use FileHandle;
use VeSqlGUI;
use Qt::isa "VeSqlGUI";
use Qt::slots 
	OnSettingsChange => [ 'const QString&' ],
	OnLoadConnection => [ 'int' ],
	OnChangeDbType => [ 'int' ],
	OnSaveConnect => [],
	OnSelectHistory => [ 'int' ],
	OnRun => [];

sub NEW
{
	my $self = shift;
	$self->SUPER::NEW(@_);
	this->{DefaultPorts}{'MySQL'} = 3306;
	this->{DefaultPorts}{'PostgreSQL'} = 5432;
	
	this->{DataLoad} = 0;
	this->{Connected} = 0;
	this->{Socket} = 0;
	HistoryListBox->clear();
	QueryTextEdit->setEnabled(0);		
	Disconnect();
	
	LoadConnectionHistory();	
	this->connect(this->{HostEdit}, SIGNAL('textChanged(const QString&)'), SLOT('OnSettingsChange(const QString&)'));
	this->connect(this->{PortEdit}, SIGNAL('textChanged(const QString&)'), SLOT('OnSettingsChange(const QString&)'));
	this->connect(this->{DbNameEdit}, SIGNAL('textChanged(const QString&)'), SLOT('OnSettingsChange(const QString&)'));
	this->connect(this->{UserNameEdit}, SIGNAL('textChanged(const QString&)'), SLOT('OnSettingsChange(const QString&)'));
	this->connect(this->{PasswordEdit}, SIGNAL('textChanged(const QString&)'), SLOT('OnSettingsChange(const QString&)'));	
	
	#this->connect(this->{HistoryListBox}, SIGNAL('clicked(QListBoxItem*)'), SLOT('OnSelectHistory(QListBoxItem*)'));
	this->connect(this->{HistoryListBox}, SIGNAL('selected(int)'), SLOT('OnSelectHistory(int)'));
	this->connect(this->{DbTypeComboBox}, SIGNAL('activated(int)'), SLOT('OnChangeDbType(int)'));
	this->connect(this->{SavedComboBox}, SIGNAL('activated(int)'), SLOT('OnLoadConnection(int)'));
	this->connect(this->{RunButton}, SIGNAL('clicked()'), SLOT('OnRun()'));
	this->connect(this->{SaveConnectButton}, SIGNAL('clicked()'), SLOT('OnSaveConnect()'));
}

sub OnSaveConnect
{
	$Host = this->{HostEdit}->text();
	$DbName = this->{DbNameEdit}->text();	
	$DbType = DbTypeComboBox->currentText();
	$Port = this->{PortEdit}->text();
	$UserName = this->{UserNameEdit}->text();	
	$Password = this->{PasswordEdit}->text();	
	
	@ConnectionList = ();
	push(@ConnectionList, "$Host:$DbName:$DbType:$Port:$UserName:$Password");
	if(-f $ENV{HOME}."/.vesql/connections")
	{
		$FilePointer = new FileHandle $ENV{HOME}."/.vesql/connections", "r";
		if($FilePointer)
		{
			while($ParseLine = $FilePointer->getline)
			{
				if($ParseLine gt "")
				{
					$ParseLine =~ s/\n//g;
					@Connection = split/:/,$ParseLine;
					if("$Host:$DbName" ne $Connection[0].":".$Connection[1])
					{
						print "Keep $ParseLine\n";
						push(@ConnectionList, $ParseLine);
					}
				}
			}
			$FilePointer->close;		
		}
	}
	$FilePointer = new FileHandle $ENV{HOME}."/.vesql/connections", "w";
	if($FilePointer)
	{
		foreach $ConnectionLine (@ConnectionList)
		{
			print "Save - $ConnectionLine\n";
			print $FilePointer $ConnectionLine."\n";
		}
		$FilePointer->close;		
	} else
	{
		print "Error: Cannot write to ".$ENV{HOME}."/.vesql/connections\n";
	}		
	
	# Update the ComboBox to include the new connection the user entered and select that entry in the ComboBox
	LoadConnectionHistory();
	for($i = 0; $i < SavedComboBox->count(); $i++)
	{
		$Compare = SavedComboBox->text($i);
		if($Compare eq "$Host:$DbName")
		{
			SavedComboBox->setCurrentItem($i);
		}		
	}		
	
	###############################################################
	# Need to set the correct driver from $DbType
	print "=====================================\nFIX THIS LINE\n";
	$DbiDbType = "mysql";
	if($DbType eq "PostgreSQL")
	{
		$DbiDbType = "Pg";
	}
	###############################################################
	
	if(this->{Socket} = DBI->connect("dbi:".$DbiDbType.":".$DbName.":".$Host,$UserName,$Password))
	{	
		this->{Connected} = 1;	
		LoadQueryHistory($Host."_".$DbName.".sql");
		QueryTextEdit->setEnabled(1);		
	} else
	{
		Qt::MessageBox::critical(this, "Error Connecting", "Could not connect to that database. Please check your login information.", &Qt::MessageBox::Ok, 0);
	}
}

sub OnRun
{
	if(this->{Connected} == 1)
	{
		$SqlCommand = QueryTextEdit->text();
		if($SqlCommand ne "")
		{
			SaveQueryHistory($SqlCommand);
		}
		
		print "SQL: $SqlCommand\n";
		$SqlRef = this->{Socket}->prepare($SqlCommand);
		$SqlRef->execute;
		
		# There is a known bug in PerlQt where Perl and Qt fight over who controls the freeing of memory when table rows, columns or cells are deleted.
		# To by-pass this bug we do not try to delete table rows, columns or cells, just blank them out.
		ClearTable();
		
		$ThisTableRow = 1;
		while($row = $SqlRef->fetchrow_hashref())
		{
			$ThisTableColumn = 1;
			if(ResultsTable->numRows() < $ThisTableRow)
			{
				ResultsTable->setNumRows($ThisTableRow);	
			}
			foreach $key (keys %$row)
			{
				if($ThisTableRow == 1)
				{					
					if(ResultsTable->numCols() < $ThisTableColumn)
					{
						ResultsTable->setNumCols($ThisTableColumn);
					}
					ResultsTable->horizontalHeader()->setLabel($ThisTableColumn - 1, $key, 50);
				}
				#print "Cell [ $key ] = ".$row->{$key}."\n";
				ResultsTable->setText($ThisTableRow - 1, $ThisTableColumn - 1, $row->{$key});
				$ThisTableColumn++;
			}
			$ThisTableRow++;
			Qt::Application::processEvents();
		}	
	}
	print "and we are done\n";
}

sub ClearTable
{
	for($r = 0; $r < ResultsTable->numRows(); $r++)
	{
		for($c = 0; $c < ResultsTable->numCols(); $c++)
		{
			if($r == 0)
			{
				ResultsTable->horizontalHeader()->setLabel($c, "", 0);	
				# hideColumn
			}
			ResultsTable->setText($r, $c, "");
		}
	}
}

sub OnChangeDbType
{
	print "sub OnChangeDbType()\n";
	if(this->{DataLoad} == 0)
	{
		print "Ok to change\n";
		$TheDbType = DbTypeComboBox->currentText();
		$NewPort = this->{DefaultPorts}{$TheDbType};
		SetPort($NewPort);
	}		
}

sub OnSettingsChange
{
	if(this->{DataLoad} == 0)
	{
		print "sub OnSettingsChange\n";
		if(this->{Connected} == 1)
		{
			CloseConnection();
		}
	}
}

sub CloseConnection
{
	print "sub CloseConnection()\n";
	this->{Connected} = 0;
	this->{Socket} = 0;
	HistoryListBox->clear();
	QueryTextEdit->clear();
	QueryTextEdit->setEnabled(0);	
}

sub Disconnect
{
	print "sub Disconnect()\n";
	this->{DataLoad} = 1;
	if(this->{Connected} == 1)
	{
		CloseConnection();
	}
	SetHost("");
	SetDbType("MySQL");	
	SetDbName("");
	SetUserName("");
	SetPassword("");
	this->{DataLoad} = 0;
}

sub SetPassword
{
	$NewPass = shift;
	print "sub SetPassword()\n";
	this->{PasswordEdit}->setText($NewPass);	
}

sub SetUserName
{
	$NewUserName = shift;
	print "sub SetUserName()\n";
	this->{UserNameEdit}->setText($NewUserName);	
}

sub SetDbName
{
	$NewDbname = shift;
	print "sub SetDbName()\n";
	this->{DbNameEdit}->setText($NewDbname);	
}

sub SetPort
{
	$NewPort = shift;
	print "sub SetPort($NewPort)\n";
	if($NewPort eq "")
	{
		$TheDbType = DbTypeComboBox->currentText();
		$NewPort = this->{DefaultPorts}{$TheDbType};
	}
	this->{PortEdit}->setText($NewPort);
}

sub SetDbType
{
	$NewDbType = shift;
	print "sub SetDbType($NewDbType)\n";
	if($NewDbType eq "")
	{
		$NewDbType = DbTypeComboBox->text(0);
	}
	$ChosenType = "";
	for($i = 0; $i < DbTypeComboBox->count(); $i++)
	{
		$Compare = DbTypeComboBox->text($i);
		if($Compare eq $NewDbType)
		{
			$ChosenType = $Compare;
			DbTypeComboBox->setCurrentItem($i);
			SetPort("");
		}		
	}	
	if($ChosenType eq "")
	{
		SetDbType("");
	}
}

sub SetHost
{
	$NewHost = shift;
	print "sub SetHost($NewHost)\n";
	this->{HostEdit}->setText($NewHost);	
}

sub OnLoadConnection
{
	print "sub OnLoadConnection()\n";
	Disconnect();
	this->{DataLoad} = 1;
	$Selected = SavedComboBox->currentText();
	if(-f $ENV{HOME}."/.vesql/connections")
	{
		$FilePointer = new FileHandle $ENV{HOME}."/.vesql/connections", "r";
		if($FilePointer)
		{
			while($ParseLine = $FilePointer->getline)
			{
				if($ParseLine gt "")
				{
					$ParseLine =~ s/\n//g;
					@Connection = split/:/,$ParseLine;
					if($Selected eq $Connection[0].":".$Connection[1])
					{
						SetHost($Connection[0]);
						SetDbName($Connection[1]);
						SetDbType($Connection[2]);	
						SetPort($Connection[3]);
						SetUserName($Connection[4]);
						SetPassword($Connection[5]);						
					}
				}
			}
			$FilePointer->close;		
		}
	}	
	this->{DataLoad} = 0;
}

sub LoadConnectionHistory
{
	print "sub LoadConnectionHistory()\n";
	this->{DataLoad} = 1;
	SavedComboBox->clear();
	SavedComboBox->insertItem("");
	if(! -d $ENV{HOME}."/.vesql/")
	{
		mkdir($ENV{HOME}."/.vesql/");
	}	
	if(-f $ENV{HOME}."/.vesql/connections")
	{
		$FilePointer = new FileHandle $ENV{HOME}."/.vesql/connections", "r";
		if($FilePointer)
		{
			while($ParseLine = $FilePointer->getline)
			{
				if($ParseLine gt "")
				{
					$ParseLine =~ s/\n//g;
					@Connection = split/:/,$ParseLine;
					SavedComboBox->insertItem($Connection[0].":".$Connection[1]);
				}
			}
			$FilePointer->close;		
		}
	}
	this->{DataLoad} = 0;
}

sub SaveQueryHistory
{
	$SqlCommand = shift;
	$Host = this->{HostEdit}->text();
	$DbName = this->{DbNameEdit}->text();		
	HistoryListBox->insertItem($SqlCommand, 0);
	$FilePointer = new FileHandle $ENV{HOME}."/.vesql/".$Host."_".$DbName.".sql", "w";
	if($FilePointer)
	{
		for(my $i = 0; $i < HistoryListBox->count(); $i++)
		{
			print $FilePointer HistoryListBox->text($i)."\n";
		}		
		$FilePointer->close;		
	}	
}


sub LoadQueryHistory
{
	$HistoryFile = shift;
	print "sub LoadQueryHistory($HistoryFile)\n";
	HistoryListBox->clear();
	if(-f $ENV{HOME}."/.vesql/".$HistoryFile)
	{
		$FilePointer = new FileHandle $ENV{HOME}."/.vesql/".$Host."_".$DbName.".sql", "r";
		if($FilePointer)
		{
			while($ParseLine = $FilePointer->getline)
			{
				if($ParseLine gt "")
				{
					$ParseLine =~ s/\n//g;
					print "HistoryListBox->insertItem($ParseLine, 0);\n";
					HistoryListBox->insertItem($ParseLine, 0);
				}
			}
			$FilePointer->close;		
		}	
	}	
}

sub OnSelectHistory
{
	$ChosenCommandIndex = shift;
	$ChosenCommand = HistoryListBox->text($ChosenCommandIndex);
	print "sub OnSelectHistory($ChosenCommand)\n";
	QueryTextEdit->setText($ChosenCommand);
}

sub WriteConf
{
   local ($FileToWrite,$VarToSet,$ValueToSet) = @_;
   local ($WriteFilePointer,$ReadFilePointer);
   local ($TempFileName);
   local ($FoundInConf);
   local ($ParseLine);
   local ($ReadVar,$ReadVal);
   local ($PostLine);

	if(! -f $FileToWrite)
	{
		$WriteFilePointer = new FileHandle $FileToWrite, "w";
		if(! $WriteFilePointer)
		{
			die("Could not write to file '$FileToWrite'\n");
		}
		print $WriteFilePointer "\n";
		$WriteFilePointer->close;
	}
	
   $TempFileName = $FileToWrite."_tmp";
   move($FileToWrite,$TempFileName);

   $FoundInConf = 0;
   $ReadFilePointer = new FileHandle $TempFileName, "r";
   $WriteFilePointer = new FileHandle $FileToWrite, "w";
   while(! $ReadFilePointer->eof)
   {
      $ParseLine = $ReadFilePointer->getline;
      $ParseLine =~ s/\n//g;
      if($ParseLine gt "")
      {
         ($ReadVar,$ReadVal) = split/ = /,$ParseLine;
         if($ReadVar eq $VarToSet)
         {
            $FoundInConf = 1;
            $PostLine = $VarToSet." = ".$ValueToSet."\n";
            print $WriteFilePointer $PostLine;
         } else
         {
            $PostLine = $ParseLine."\n";
            if($ParseLine gt "")
            {
               print $WriteFilePointer $PostLine;
            }
         }
      }
   }
   $ReadFilePointer->close;
   if($FoundInConf == 0)
   {
      $PostLine = $VarToSet." = ".$ValueToSet."\n";
      print $WriteFilePointer $PostLine;
   }
   $WriteFilePointer->close;

   if(-f $TempFileName)
   {
      unlink($TempFileName);
   }
}

sub ReadConfDef
{
	$FileToRead = shift;
	$VarToGet = shift;
	$DefaultValue = shift;
	$TempReturn = "";
	
	$TempReturn = ReadConf($FileToRead, $VarToGet);
	if($TempReturn eq "")
	{
		$TempReturn = $DefaultValue;
		WriteConf($FileToRead, $VarToGet, $DefaultValue);
	}
	return($TempReturn);
}

sub ReadConf
{
	$FileToRead = shift;
	$VarToGet = shift;
	
	$FilePointer = 0;
	$ReadVar = "";
	$ReadVal = "";
	$DoneLooking = 0;
	$TempReturn = "";
	
	$FilePointer = new FileHandle $FileToRead, "r";
	if($FilePointer)
	{
		while($DoneLooking == 0)
		{
			$ParseLine = $FilePointer->getline;
			if($ParseLine gt "")
			{
				$ParseLine =~ s/\n//g;
				($ReadVar,$ReadVal) = split/ = /,$ParseLine;
				if($ReadVar eq $VarToGet)
				{
					$DoneLooking = 1;
					$TempReturn = $ReadVal;
				}
			}
			if($FilePointer->eof)
			{
				$DoneLooking = 1;
			}
		}
		$FilePointer->close;
	}
	return($TempReturn);
}

1;
